@@ -15,6 +15,8 @@
 
 import sys
 from copy import deepcopy
+from typing import Union, Tuple
+
 import numpy as np
 import SimpleITK as sitk
 from batchgenerators.augmentations.utils import resize_segmentation
@@ -22,10 +24,13 @@ from nnunet.preprocessing.preprocessing import get_lowres_axis, get_do_separate_
 from batchgenerators.utilities.file_and_folder_operations import *
 
 
-def save_segmentation_nifti_from_softmax(segmentation_softmax, out_fname, dct, order=1, region_class_order=None,
-                                         seg_postprogess_fn=None, seg_postprocess_args=None, resampled_npz_fname=None,
-                                         non_postprocessed_fname=None, force_separate_z=None,
-                                         interpolation_order_z=0):
+def save_segmentation_nifti_from_softmax(segmentation_softmax: Union[str, np.ndarray], out_fname: str,
+                                         properties_dict: dict, order: int = 1,
+                                         region_class_order: Tuple[Tuple[int]] = None,
+                                         seg_postprogess_fn: callable = None, seg_postprocess_args: tuple = None,
+                                         resampled_npz_fname: str = None,
+                                         non_postprocessed_fname: str = None, force_separate_z: bool = None,
+                                         interpolation_order_z: int = 0):
     """
     This is a utility for writing segmentations to nifto and npz. It requires the data to have been preprocessed by
     GenericPreprocessor because it depends on the property dictionary output (dct) to know the geometry of the original
@@ -43,7 +48,7 @@ def save_segmentation_nifti_from_softmax(segmentation_softmax, out_fname, dct, o
     filename or np.ndarray for segmentation_softmax and will handle this automatically
     :param segmentation_softmax:
     :param out_fname:
-    :param dct:
+    :param properties_dict:
     :param order:
     :param region_class_order:
     :param seg_postprogess_fn:
@@ -65,26 +70,26 @@ def save_segmentation_nifti_from_softmax(segmentation_softmax, out_fname, dct, o
 
     # first resample, then put result into bbox of cropping, then save
     current_shape = segmentation_softmax.shape
-    shape_original_after_cropping = dct.get('size_after_cropping')
-    shape_original_before_cropping = dct.get('original_size_of_raw_data')
+    shape_original_after_cropping = properties_dict.get('size_after_cropping')
+    shape_original_before_cropping = properties_dict.get('original_size_of_raw_data')
     # current_spacing = dct.get('spacing_after_resampling')
     # original_spacing = dct.get('original_spacing')
 
     if np.any([i != j for i, j in zip(np.array(current_shape[1:]), np.array(shape_original_after_cropping))]):
         if force_separate_z is None:
-            if get_do_separate_z(dct.get('original_spacing')):
+            if get_do_separate_z(properties_dict.get('original_spacing')):
                 do_separate_z = True
-                lowres_axis = get_lowres_axis(dct.get('original_spacing'))
-            elif get_do_separate_z(dct.get('spacing_after_resampling')):
+                lowres_axis = get_lowres_axis(properties_dict.get('original_spacing'))
+            elif get_do_separate_z(properties_dict.get('spacing_after_resampling')):
                 do_separate_z = True
-                lowres_axis = get_lowres_axis(dct.get('spacing_after_resampling'))
+                lowres_axis = get_lowres_axis(properties_dict.get('spacing_after_resampling'))
             else:
                 do_separate_z = False
                 lowres_axis = None
         else:
             do_separate_z = force_separate_z
             if do_separate_z:
-                lowres_axis = get_lowres_axis(dct.get('original_spacing'))
+                lowres_axis = get_lowres_axis(properties_dict.get('original_spacing'))
             else:
                 lowres_axis = None
 
@@ -92,14 +97,14 @@ def save_segmentation_nifti_from_softmax(segmentation_softmax, out_fname, dct, o
         seg_old_spacing = resample_data_or_seg(segmentation_softmax, shape_original_after_cropping, is_seg=False,
                                                axis=lowres_axis, order=order, do_separate_z=do_separate_z, cval=0,
                                                order_z=interpolation_order_z)
-        #seg_old_spacing = resize_softmax_output(segmentation_softmax, shape_original_after_cropping, order=order)
+        # seg_old_spacing = resize_softmax_output(segmentation_softmax, shape_original_after_cropping, order=order)
     else:
         print("no resampling necessary")
         seg_old_spacing = segmentation_softmax
 
     if resampled_npz_fname is not None:
         np.savez_compressed(resampled_npz_fname, softmax=seg_old_spacing.astype(np.float16))
-        save_pickle(dct, resampled_npz_fname[:-4] + ".pkl")
+        save_pickle(properties_dict, resampled_npz_fname[:-4] + ".pkl")
 
     if region_class_order is None:
         seg_old_spacing = seg_old_spacing.argmax(0)
@@ -109,7 +114,7 @@ def save_segmentation_nifti_from_softmax(segmentation_softmax, out_fname, dct, o
             seg_old_spacing_final[seg_old_spacing[i] > 0.5] = c
         seg_old_spacing = seg_old_spacing_final
 
-    bbox = dct.get('crop_bbox')
+    bbox = properties_dict.get('crop_bbox')
 
     if bbox is not None:
         seg_old_size = np.zeros(shape_original_before_cropping)
@@ -127,16 +132,16 @@ def save_segmentation_nifti_from_softmax(segmentation_softmax, out_fname, dct, o
         seg_old_size_postprocessed = seg_old_size
 
     seg_resized_itk = sitk.GetImageFromArray(seg_old_size_postprocessed.astype(np.uint8))
-    seg_resized_itk.SetSpacing(dct['itk_spacing'])
-    seg_resized_itk.SetOrigin(dct['itk_origin'])
-    seg_resized_itk.SetDirection(dct['itk_direction'])
+    seg_resized_itk.SetSpacing(properties_dict['itk_spacing'])
+    seg_resized_itk.SetOrigin(properties_dict['itk_origin'])
+    seg_resized_itk.SetDirection(properties_dict['itk_direction'])
     sitk.WriteImage(seg_resized_itk, out_fname)
 
     if (non_postprocessed_fname is not None) and (seg_postprogess_fn is not None):
         seg_resized_itk = sitk.GetImageFromArray(seg_old_size.astype(np.uint8))
-        seg_resized_itk.SetSpacing(dct['itk_spacing'])
-        seg_resized_itk.SetOrigin(dct['itk_origin'])
-        seg_resized_itk.SetDirection(dct['itk_direction'])
+        seg_resized_itk.SetSpacing(properties_dict['itk_spacing'])
+        seg_resized_itk.SetOrigin(properties_dict['itk_origin'])
+        seg_resized_itk.SetDirection(properties_dict['itk_direction'])
         sitk.WriteImage(seg_resized_itk, non_postprocessed_fname)
 
 
@@ -157,7 +162,7 @@ def save_segmentation_nifti(segmentation, out_fname, dct, order=1, force_separat
 
     if isinstance(segmentation, str):
         assert isfile(segmentation), "If isinstance(segmentation_softmax, str) then " \
-                                             "isfile(segmentation_softmax) must be True"
+                                     "isfile(segmentation_softmax) must be True"
         del_file = deepcopy(segmentation)
         segmentation = np.load(segmentation)
         os.remove(del_file)
@@ -190,9 +195,10 @@ def save_segmentation_nifti(segmentation, out_fname, dct, order=1, force_separat
                 else:
                     lowres_axis = None
 
-            print("separate z:",do_separate_z, "lowres axis", lowres_axis)
+            print("separate z:", do_separate_z, "lowres axis", lowres_axis)
             seg_old_spacing = resample_data_or_seg(segmentation[None], shape_original_after_cropping, is_seg=True,
-                                                   axis=lowres_axis, order=order, do_separate_z=do_separate_z, cval=0)[0]
+                                                   axis=lowres_axis, order=order, do_separate_z=do_separate_z, cval=0)[
+                0]
     else:
         seg_old_spacing = segmentation
 
@@ -214,4 +220,4 @@ def save_segmentation_nifti(segmentation, out_fname, dct, order=1, force_separat
     seg_resized_itk.SetDirection(dct['itk_direction'])
     sitk.WriteImage(seg_resized_itk, out_fname)
 
-    sys.stdout = sys.__stdout__
\ No newline at end of file
+    sys.stdout = sys.__stdout__
