@@ -1,5 +1,23 @@
+#    Copyright 2020 Division of Medical Image Computing, German Cancer Research Center (DKFZ), Heidelberg, Germany
+#
+#    Licensed under the Apache License, Version 2.0 (the "License");
+#    you may not use this file except in compliance with the License.
+#    You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS,
+#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#    See the License for the specific language governing permissions and
+#    limitations under the License.
+
+
 from multiprocessing.pool import Pool
+from time import sleep
+
 import matplotlib
+from nnunet.postprocessing.connected_components import determine_postprocessing
 from nnunet.training.data_augmentation.default_data_augmentation import get_default_augmentation
 from nnunet.training.dataloading.dataset_loading import DataLoader3D, unpack_dataset
 from nnunet.evaluation.evaluator import aggregate_scores
@@ -35,10 +53,8 @@ class nnUNetTrainerCascadeFullRes(nnUNetTrainer):
             self.folder_with_segs_from_prev_stage = folder_with_segs_prev_stage
             # Do not put segs_prev_stage into self.output_folder as we need to unpack them for performance and we
             # don't want to do that in self.output_folder because that one is located on some network drive.
-            self.folder_with_segs_from_prev_stage_for_train = join(self.dataset_directory, "segs_prev_stage")
         else:
             self.folder_with_segs_from_prev_stage = None
-            self.folder_with_segs_from_prev_stage_for_train = None
 
     def do_split(self):
         super(nnUNetTrainerCascadeFullRes, self).do_split()
@@ -71,11 +87,23 @@ class nnUNetTrainerCascadeFullRes(nnUNetTrainer):
         self.num_input_channels += (self.num_classes - 1)  # for seg from prev stage
 
     def setup_DA_params(self):
-        super(nnUNetTrainerCascadeFullRes, self).setup_DA_params()
-        self.data_aug_params['selected_seg_channels'] = [0, 1]
-        self.data_aug_params['all_segmentation_labels'] = list(range(1, self.num_classes))
+        super().setup_DA_params()
         self.data_aug_params['move_last_seg_chanel_to_data'] = True
-        self.data_aug_params['advanced_pyramid_augmentations'] = True
+        self.data_aug_params['cascade_do_cascade_augmentations'] = True
+
+        self.data_aug_params['cascade_random_binary_transform_p'] = 0.4
+        self.data_aug_params['cascade_random_binary_transform_p_per_label'] = 1
+        self.data_aug_params['cascade_random_binary_transform_size'] = (1, 8)
+
+        self.data_aug_params['cascade_remove_conn_comp_p'] = 0.2
+        self.data_aug_params['cascade_remove_conn_comp_max_size_percent_threshold'] = 0.15
+        self.data_aug_params['cascade_remove_conn_comp_fill_with_other_class_p'] = 0.0
+
+        # we have 2 channels now because the segmentation from the previous stage is stored in 'seg' as well until it
+        # is moved to 'data' at the end
+        self.data_aug_params['selected_seg_channels'] = [0, 1]
+        # needed for converting the segmentation from the previous stage to one hot
+        self.data_aug_params['all_segmentation_labels'] = list(range(1, self.num_classes))
 
     def initialize(self, training=True, force_load_plans=False):
         """
@@ -94,24 +122,6 @@ class nnUNetTrainerCascadeFullRes(nnUNetTrainer):
         self.folder_with_preprocessed_data = join(self.dataset_directory, self.plans['data_identifier'] +
                                                   "_stage%d" % self.stage)
         if training:
-            # copy segs from prev stage to separate folder and extract them
-
-            # If we don't do this then we need to make sure to manually delete the folder if we want to update
-            # segs_from_prev_stage. I will probably forget to do so, so I leave this in as a safeguard
-            if isdir(self.folder_with_segs_from_prev_stage_for_train):
-                shutil.rmtree(self.folder_with_segs_from_prev_stage_for_train)
-
-            maybe_mkdir_p(self.folder_with_segs_from_prev_stage_for_train)
-            segs_from_prev_stage_files = subfiles(self.folder_with_segs_from_prev_stage, suffix='.npz')
-            for s in segs_from_prev_stage_files:
-                shutil.copy(s, self.folder_with_segs_from_prev_stage_for_train)
-
-            # if we don't do this then performance is shit
-            if self.unpack_data:
-                unpack_dataset(self.folder_with_segs_from_prev_stage_for_train)
-
-            self.folder_with_segs_from_prev_stage = self.folder_with_segs_from_prev_stage_for_train
-
             self.setup_DA_params()
 
             if self.folder_with_preprocessed_data is not None:
@@ -133,12 +143,13 @@ class nnUNetTrainerCascadeFullRes(nnUNetTrainer):
                 self.print_to_log_file("VALIDATION KEYS:\n %s" % (str(self.dataset_val.keys())))
         else:
             pass
-        self.initialize_network_optimizer_and_scheduler()
+        self.initialize_network()
         assert isinstance(self.network, SegmentationNetwork)
         self.was_initialized = True
 
     def validate(self, do_mirroring=True, use_train_mode=False, tiled=True, step=2, save_softmax=True,
-                 use_gaussian=True, validation_folder_name='validation'):
+                 use_gaussian=True, overwrite=True, validation_folder_name="validation_raw",
+                 debug=False):
         """
 
         :param do_mirroring:
@@ -207,12 +218,12 @@ class nnUNetTrainerCascadeFullRes(nnUNetTrainer):
             patching system python code. We circumvent that problem here by saving softmax_pred to a npy file that will 
             then be read (and finally deleted) by the Process. save_segmentation_nifti_from_softmax can take either 
             filename or np.ndarray and will handle this automatically"""
-            if np.prod(softmax_pred.shape) > (2e9 / 4 * 0.9): # *0.9 just to be save
+            if np.prod(softmax_pred.shape) > (2e9 / 4 * 0.85):  # *0.85 just to be save
                 np.save(fname + ".npy", softmax_pred)
                 softmax_pred = fname + ".npy"
             results.append(process_manager.starmap_async(save_segmentation_nifti_from_softmax,
                                                          ((softmax_pred, join(output_folder, fname + ".nii.gz"),
-                                                           properties, 1, None, None, None, softmax_fname, None),
+                                                           properties, 3, None, None, None, softmax_fname, None),
                                                           )
                                                          )
                            )
@@ -228,3 +239,30 @@ class nnUNetTrainerCascadeFullRes(nnUNetTrainer):
                              json_output_file=join(output_folder, "summary.json"), json_name=job_name,
                              json_author="Fabian", json_description="",
                              json_task=task)
+
+        # in the old nnunet we would stop here. Now we add a postprocessing. This postprocessing can remove everything
+        # except the largest connected component for each class. To see if this improves results, we do this for all
+        # classes and then rerun the evaluation. Those classes for which this resulted in an improved dice score will
+        # have this applied during inference as well
+        self.print_to_log_file("determining postprocessing")
+        determine_postprocessing(self.output_folder, self.gt_niftis_folder, validation_folder_name,
+                                 final_subf_name=validation_folder_name + "_postprocessed", debug=debug)
+        # after this the final predictions for the vlaidation set can be found in validation_folder_name_base + "_postprocessed"
+        # They are always in that folder, even if no postprocessing as applied!
+
+        # detemining postprocesing on a per-fold basis may be OK for this fold but what if another fold finds another
+        # postprocesing to be better? In this case we need to consolidate. At the time the consolidation is going to be
+        # done we won't know what self.gt_niftis_folder was, so now we copy all the niftis into a separate folder to
+        # be used later
+        gt_nifti_folder = join(self.output_folder_base, "gt_niftis")
+        maybe_mkdir_p(gt_nifti_folder)
+        for f in subfiles(self.gt_niftis_folder, suffix=".nii.gz"):
+            success = False
+            attempts = 0
+            while not success and attempts < 10:
+                try:
+                    shutil.copy(f, gt_nifti_folder)
+                    success = True
+                except OSError:
+                    attempts += 1
+                    sleep(1)
